#!/bin/sh
help="$(cat<<EOF
Generate snapshot tarball from git repository.

Syntax:
  sh $0 [<options>]
    Generate a snapshot tarball

Options:
  --update-config
    Update spec file, README.foo, etc. with the correct timestamps,
    commit ids, etc.
  --skip-pull
    Do not fetch updates from upstream repository.
  --no-checks
    Disable check runs.

Requires:
  git, automake, autoconf, libtool
EOF
)"

# Local settings
name="xorg-x11-drv-radeonhd"
tarbase="xf86-video-radeonhd"
builddir="${tarbase}.build"
srcdir="${tarbase}.src"
specfile="${name}.spec"
date="${date-$(date +%Y%m%d)}"

branch="fedora/snapshot"

# Check files and directories before starting to do anything
if [ -d "${builddir}" ]; then
    echo "Remove the \"${builddir}\" directory first !"
    exit 1
fi

# Parse command line options
opt_update_config="false"
opt_skip_pull="false"
opt_no_checks="false"
for opt
do
    case "$opt" in
	--skip-pull)      opt_skip_pull=":" ;;
	--update-config)  opt_update_config=":" ;;
	--no-checks)      opt_no_checks=":" ;;
	-h|--help)
	    echo "$help"
	    exit 0
	    ;;
	*)
	    echo "FATAL: Unhandled command line option: $opt"
	    exit 13
	    ;;
    esac
done


# Syntax: sed_update_cond <FILENAME> [<sed-options>]
sed_update_cond() {
    old="$1"
    new="$1.new"
    shift
    if sed "$@" < "${old}" > "${new}"; then
	if cmp "${new}" "${old}"; then :
	else
	    mv -f "${new}" "${old}"
	    return
	fi
    fi
    rm -f "${new}"
}


# Prepare for action
unset CDPATH
set -ex

# Update/check out source
if [ -s "${srcdir}/.git/refs/heads/master" ]; then
    ( cd "${srcdir}" && git reset --hard HEAD && git checkout master )
    if "$opt_skip_pull"; then
	echo "Skipping pull from upstream as requested"
    else
        # update from repo, keeping local changes separate
	( cd "${srcdir}" && git fetch && git rebase origin ) || exit $?
    fi
    ( cd "${srcdir}" && (git branch -D "${branch}" ||:))
else
    # first checkout
    git clone "git://anongit.freedesktop.org/git/xorg/driver/${tarbase}" "${srcdir}"
fi
( cd "${srcdir}" && git checkout -b "${branch}" )

# Determine latest upstream commit:
upstream_commit="$(cd "${srcdir}" && git log | head -n1 | cut -d' ' -f2)"

# This patch is waiting for the time when we actually need a radeonhd.xinf file.
#    ${name}-Generate-xinf-file-for-hwdata-from-rhd_id.c.patch

# Fix and/or clean up upstream's git repo before creating a dist tarball:
#  * upstream tends to forget to make sure the dist tarball builds
#  * we avoid ugly simultanous spec patches to Makefile.{am,in}
for patch in
do
    if [ -s "$patch" ]; then
	(cd "$srcdir" && git-am -3) < "$patch"
	if test -s "${srcdir}/.dotest/patch"; then
	    echo "Fatal: Could not cleanly apply patch \"$patch\"."
	    exit 11
	fi
    else
	echo "Fatal: Patch file \"$patch\" not found."
	exit 13
    fi
done

# Generate and configure build system
autoreconf -vis "${srcdir}"
mkdir "${builddir}"
( cd "${builddir}" && ../${srcdir}/configure )

# Build dist tarball and move it to CWD
version=`sed -n '/^PACKAGE_VERSION = / { s/^PACKAGE_VERSION = //;p }' "${builddir}/Makefile"`
tarname="${tarbase}-${version}-${date}git"
if "$opt_no_checks"; then
    disttarget="dist"
else
    disttarget="distcheck"
fi
make -C "${builddir}" "${disttarget}" distdir="${tarname}"
mv -f "${builddir}/${tarname}.tar.bz2" "${tarname}.tar.bz2"

# Update spec file with the proper timestamp
if "$opt_update_config"; then
    sed_update_cond "$specfile" \
	-e "s/^%define git_commit .*/%define git_commit ${upstream_commit}/" \
	-e "s/^%define date .*/%define date ${date}/" \
        -e "s/^Version:.*/Version:       ${version}/"

    sed_update_cond "${name}-README.fedora" \
	-e "s/^\(Latest upstream commit:\).*/\\1 ${upstream_commit}/"
fi

# Clean up after us (keeping the patched git branch)
rm -rf "${builddir}"

echo "Latest upstream commit: ${upstream_commit}"
echo "$ make new-sources FILES=\"${tarname}.tar.bz2\""

